#include <stdio.h>
#include "opencv2/opencv.hpp"

#define PIXVAL(iplimagep, x, y) (*(uchar *)((iplimagep)->imageData + (y) * (iplimagep)->widthStep + (x)))

// smoothing (approx. Gaussian)
int denom = 8;
int w[3 * 3] = { 0,  1,  0, 
                 1,  4,  1,
                 0,  1,  0 }; 

/*
// horizontal edge (Sobel)
#define RESULT_SIGNED
int denom = 8;
int w[3 * 3] = { -1,  -2,  -1, 
                  0,  0,  0,
                  1,  2,  1 }; 
*/

/*
// vertical edge (Sobel)
#define RESULT_SIGNED
int denom = 1;
int w[3 * 3] = { -1,  0,  1, 
                 -2,  0,  2,
                 -1,  0,  1 }; 
*/

/*
// high-pass (approx. Laplacian)
#define RESULT_SIGNED
int denom = 1;
int w[3 * 3] = { 0,  1,  0, 
                 1, -4,  1,
                 0,  1,  0 };
*/

/*
// sharpening (Identity - approx. Laplacian)
int denom = 1;
int w[3 * 3] = { 0,  -1,  0, 
                -1,  5,  -1,
                 0,  -1,  0 }; 
*/

#ifdef RESULT_SIGNED
#define RESULT_TYPE IPL_DEPTH_8S
#define SAT_MAX 127
#define SAT_MIN -128
#else
#define RESULT_TYPE IPL_DEPTH_8U
#define SAT_MAX 255
#define SAT_MIN 0
#endif

uchar saturate(int v)
{
    if (v >= SAT_MAX) {
        return SAT_MAX;
    } else if (v <= SAT_MIN) {
        return SAT_MIN;
    } else {
        return (uchar)v;
    }
}

void
filter3x3(IplImage *img, IplImage *result)
{
    int i, j, m, n;

    for (i = 0; i < img->width; i++) { /* first row */
        PIXVAL(result, i, 0) = 0;
    }

    for (j = 1; j < img->height - 1; j++) {
        PIXVAL(result, 0, j) = 0; /* leftmost pixel */
        for (i = 1; i < img->width - 1; i++) {
            int sum = 0;
            for (n = 0; n < 3; n++) {
                for (m = 0; m < 3; m++) {
                    sum += w[m + 3 * n] * PIXVAL(img, i - 1 + m, j - 1 + n);
                }
            }
            PIXVAL(result, i, j) = saturate(sum / denom);
        }
        PIXVAL(result, img->width - 1, j) = 0; /* rightmost pixel */
    }

    for (i = 0; i < img->width; i++) { /* last row */
        PIXVAL(result, i, img->height - 1) = 0;
    }

}

int
main()
{
    IplImage *frame, *img, *result;
    CvCapture *capture = cvCaptureFromCAM(0);
    if (capture == NULL) {
        fprintf(stderr, "cannot capture\n");
        return 1;
    }

    frame = cvQueryFrame(capture);
    img = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);
    result = cvCreateImage(cvGetSize(frame), RESULT_TYPE, 1);

    cvNamedWindow("result", CV_WINDOW_AUTOSIZE);
    cvNamedWindow("original", CV_WINDOW_AUTOSIZE);

    while (1) {
        frame = cvQueryFrame(capture); /* cannot be modified */

        cvCvtColor(frame, img, CV_BGR2GRAY);

        filter3x3(img, result);

	cvCvtColor(frame, img, CV_BGR2GRAY);
        cvShowImage("original", img);
        cvShowImage("result", result);

        if (cvWaitKey(30) > 0) {
            break;
        }
    }

    return 0;
}
