#include <stdio.h>
#include "opencv2/opencv.hpp"

#define PIXVAL(iplimagep, x, y) (*(uchar *)((iplimagep)->imageData + (y) * (iplimagep)->widthStep + (x)))

void
dilation(IplImage *src, IplImage *dst)
{
    int i, j;

    for (i = 0; i < src->width; i++) { /* first row */
        PIXVAL(dst, i, 0) = 0;
    }
    for (j = 1; j < src->height - 1; j++) {
        PIXVAL(dst, 0, j) = 0; /* leftmost pixel */
        for (i = 1; i < src->width - 1; i++) {
            PIXVAL(dst, i, j) 
                = 255 * (PIXVAL(src, i, j) 
                         || PIXVAL(src, i - 1, j)
                         || PIXVAL(src, i + 1, j)
                         || PIXVAL(src, i, j - 1)
                         || PIXVAL(src, i, j + 1));
        }
        PIXVAL(dst, src->width - 1, j) = 0; /* rightmost pixel */
    }
    for (i = 0; i < src->width; i++) { /* last row */
        PIXVAL(dst, i, src->height - 1) = 0;
    }
}

void
erosion(IplImage *src, IplImage *dst)
{
    int i, j;

    for (i = 0; i < src->width; i++) { /* first row */
        PIXVAL(dst, i, 0) = 0;
    }
    for (j = 1; j < src->height - 1; j++) {
        PIXVAL(dst, 0, j) = 0; /* leftmost pixel */
        for (i = 1; i < src->width - 1; i++) {
            PIXVAL(dst, i, j) 
                = 255 * (PIXVAL(src, i, j) 
                         && PIXVAL(src, i - 1, j)
                         && PIXVAL(src, i + 1, j)
                         && PIXVAL(src, i, j - 1)
                         && PIXVAL(src, i, j + 1));
        }
        PIXVAL(dst, src->width - 1, j) = 0; /* rightmost pixel */
    }
    for (i = 0; i < src->width; i++) { /* last row */
        PIXVAL(dst, i, src->height - 1) = 0;
    }
}

int main()
{
    int i, j;

    CvCapture *capture = cvCaptureFromCAM(0);

    IplImage *frame, *img, *bin, *result, *result2;

    if (capture == NULL) {
        fprintf(stderr, "cannot capture\n");
        return 1;
    }

    frame = cvQueryFrame(capture);
    img = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);
    bin = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);
    result = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);
    result2 = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);

    cvNamedWindow("input", CV_WINDOW_AUTOSIZE);
    cvNamedWindow("binary", CV_WINDOW_AUTOSIZE);
    cvNamedWindow("result", CV_WINDOW_AUTOSIZE);

    while (1) {
        frame = cvQueryFrame(capture); /* cannot be modified */

        cvCvtColor(frame, img, CV_BGR2GRAY);

        for (j = 0; j < img->height; j++) {
            for (i = 0; i < img->width; i++) {
                PIXVAL(bin, i, j) = (PIXVAL(img, i, j) > 128)? 255: 0;
            }
        }

        erosion(bin, result);
        /*
        erosion(result, result2);
        erosion(result2, result);
        */

        dilation(result, result2);
	/*
        dilation(result2, result);
        dilation(result, result2);
        */
        cvCopy(result2, result);

        cvShowImage("input", img);
        cvShowImage("binary", bin);
        cvShowImage("result", result);

        if (cvWaitKey(30) > 0) {
            break;
        }
    }

    return 0;
}
